"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Runnable = void 0;
class Runnable {
    constructor(id, task) {
        this.assertInit(id, task);
        this.id = id;
        this.task = task;
    }
    assertInit(id, task) {
        if (id === undefined || id === null) {
            throw new Error('runnable:id:missing');
        }
        if (typeof id !== 'string') {
            throw new Error('runnable:id:bad_format');
        }
        if (task === undefined || task === null) {
            throw new Error('runnable:task:missing');
        }
        if (typeof task !== 'function') {
            throw new Error('runnable:task:bad_format');
        }
    }
    run(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const outcome = {
                execution: {
                    exception: false,
                    complete: false,
                    errors: []
                },
                returnValue: null
            };
            try {
                outcome.returnValue = yield Promise.resolve(this.task(data));
                outcome.execution.complete = true;
                outcome.execution.exception = false;
            }
            catch (e) {
                if (e instanceof Error) {
                    outcome.execution.errors.push(e);
                }
                outcome.returnValue = null;
                outcome.execution.complete = false;
                outcome.execution.exception = true;
            }
            return outcome;
        });
    }
}
exports.Runnable = Runnable;
